package com.yideb.audit.controller;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.yideb.audit.config.GlobalConstant;
import com.yideb.audit.entity.audit.vo.BasicVo;
import com.yideb.audit.service.audit.IBottomService;
import com.yideb.audit.service.audit.IStatisticConfigService;
import com.yideb.audit.service.hub.IPrescriptionTrendService;
import com.yideb.audit.service.hub.IStoreTrendService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <b>Application name：</b> yideb-audit-statistic <br>
 * <b>Application describing： </b> <br>
 * <b>Copyright：</b> Copyright &copy; 2020 明医众禾科技（北京）有限责任公司 版权所有。<br>
 * <b>Company：</b> 明医众禾科技（北京）有限责任公司 <br>
 * <b>@Date：</b> 2020/8/10 16:34 <br>
 * <b>@Author：</b> <a href="mailto:lijing@miyzh.com"> 李璟 </a> <br>
 * <b>@Checker：</b>
 */
@Slf4j
@RestController
@RequestMapping("daily")
public class DailyHistoryController {

	@Resource
	private IBottomService bottomService;
	@Resource
	private IStatisticConfigService statisticConfigService;
	@Resource
	private IStoreTrendService storeTrendService;
	@Resource
	private IPrescriptionTrendService prescriptionTrendService;

	@GetMapping("dailyPrescriptions")
	public String dailyPrescriptions() {
		log.info("dailyPrescriptions 处理开始");
		String endTime = DateUtil.beginOfDay(DateUtil.yesterday()).toString(DatePattern.NORM_DATETIME_PATTERN);
		List<BasicVo> dailyPrescriptions = bottomService.dailyPrescriptions(endTime);

		// 插入hub数据库
		Integer prescriptionAddDayMultiple = getConfig().getOrDefault(GlobalConstant.BusinessType.AUDIT_ADD_DAY.getCode(), 1);

		prescriptionTrendService.batchInsert(dailyPrescriptions, prescriptionAddDayMultiple);
		log.info("dailyPrescriptions 处理结束");

		return "success";
	}

	@GetMapping("dailyStores")
	public String dailyStores() {
		log.info("dailyStores 处理开始");
		String endTime = DateUtil.beginOfDay(DateUtil.yesterday()).toString(DatePattern.NORM_DATETIME_PATTERN);
		List<BasicVo> dailyStores = bottomService.dailyStores(endTime);

		// 插入hub数据库
		Integer storeAddDayMultiple = getConfig().getOrDefault(GlobalConstant.BusinessType.STORE_ADD_DAY.getCode(), 1);
		storeTrendService.batchInsert(dailyStores, storeAddDayMultiple);
		log.info("dailyStores 处理结束");

		return "success";
	}

	private Map<Integer, Integer> getConfig() {
		return statisticConfigService.allConfigs();
	}
}
