package com.yideb.audit.task;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.yideb.audit.config.GlobalConstant;
import com.yideb.audit.entity.audit.vo.BasicVo;
import com.yideb.audit.entity.audit.vo.MapVo;
import com.yideb.audit.service.audit.IBottomService;
import com.yideb.audit.service.audit.IStatisticConfigService;
import com.yideb.audit.service.hub.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <b>Application name：</b> yideb-audit-statistic <br>
 * <b>Application describing： </b> <br>
 * <b>Copyright：</b> Copyright &copy; 2020 明医众禾科技（北京）有限责任公司 版权所有。<br>
 * <b>Company：</b> 明医众禾科技（北京）有限责任公司 <br>
 * <b>@Date：</b> 2020/8/10 15:34 <br>
 * <b>@Author：</b> <a href="mailto:lijing@miyzh.com"> 李璟 </a> <br>
 * <b>@Checker：</b>
 */
@Slf4j
@Component
public class BottomScheduleTask {

	@Resource
	private IBottomService bottomService;
	@Resource
	private IStatisticConfigService statisticConfigService;
	@Resource
	private IStoreTrendService storeTrendService;
	@Resource
	private IPrescriptionTrendService prescriptionTrendService;
	@Resource
	private IMedicineRankService medicineRankService;
	@Resource
	private IPrescriptionRankService prescriptionRankService;
	@Resource
	private IAreaStoreService areaStoreService;

	@Scheduled(cron = "0 0 3 * * ?")
	public void execute() {
		log.info("底部数据刷新任务开始");
		long startTime = System.currentTimeMillis();

		// 读取数据库配置
		log.info("读取配置表信息开始");
		Map<Integer, Integer> configMap = statisticConfigService.allConfigs();
		Integer storeAddDayMultiple = configMap.getOrDefault(GlobalConstant.BusinessType.STORE_ADD_DAY.getCode(), 1);
		Integer prescriptionAddDayMultiple = configMap.getOrDefault(GlobalConstant.BusinessType.AUDIT_ADD_DAY.getCode(), 1);
		Integer storeTopMultiple = configMap.getOrDefault(GlobalConstant.BusinessType.STORE_TOP_10.getCode(), 1);
		Integer storeTopLevel = configMap.getOrDefault(GlobalConstant.BusinessType.STORE_TOP_LEVEL.getCode(), GlobalConstant.DEFAULT_TOP_NUMBER);
		Integer medicineTopMultiple = configMap.getOrDefault(GlobalConstant.BusinessType.MEDICINE_TOP_10.getCode(), 1);
		Integer medicineTopLevel = configMap.getOrDefault(GlobalConstant.BusinessType.MEDICINE_TOP_LEVEL.getCode(), GlobalConstant.DEFAULT_TOP_NUMBER);
		Integer areaStoreMultiple = configMap.getOrDefault(GlobalConstant.BusinessType.AREA_STORE.getCode(), 1);
		log.info("读取配置表信息结束");
		log.info("读取配置表，花费时间：{}", System.currentTimeMillis() - startTime);

		// 昨天开始时间
		String yesterdayStartTime = DateUtil.beginOfDay(DateUtil.yesterday()).toString(DatePattern.NORM_DATETIME_PATTERN);
		String yesterdayEndTime = DateUtil.endOfDay(DateUtil.yesterday()).toString(DatePattern.NORM_DATETIME_PATTERN);

		log.info("查询底部信息开始");
		// 昨日新增药店
		List<BasicVo> lastDayStores = bottomService.lastDayStores(yesterdayStartTime, yesterdayEndTime);
		// 昨日新增处方
		List<BasicVo> lastDayPrescriptions = bottomService.lastDayPrescriptions(yesterdayStartTime, yesterdayEndTime);
		// 处方药销售排名
		List<BasicVo> storeRankList = bottomService.storeRank(storeTopLevel);
		// 药品排名
		List<BasicVo> medicineRankList = bottomService.medicineRank(medicineTopLevel);
		// 地图数据
		List<MapVo> storeMapList = bottomService.storeMap();
		log.info("查询底部信息结束");
		log.info("查询底部信息，花费时间：{}", System.currentTimeMillis() - startTime);

		// 昨日新增药店保存
		log.info("写数据到远程数据库开始");
		if (CollectionUtil.isEmpty(lastDayStores)) {
			lastDayStores = new ArrayList<>();
			BasicVo vo = new BasicVo();
			vo.setKey(DateUtil.yesterday().toString(DatePattern.NORM_DATE_PATTERN));
			vo.setValue(GlobalConstant.YesOrNo.NO.getCode());
			lastDayStores.add(vo);
		}
		storeTrendService.insert(lastDayStores, storeAddDayMultiple);
		// 昨日新增处方
		if (CollectionUtil.isEmpty(lastDayPrescriptions)) {
			lastDayPrescriptions = new ArrayList<>();
			BasicVo vo = new BasicVo();
			vo.setKey(DateUtil.yesterday().toString(DatePattern.NORM_DATE_PATTERN));
			vo.setValue(GlobalConstant.YesOrNo.NO.getCode());
			lastDayPrescriptions.add(vo);
		}
		prescriptionTrendService.insert(lastDayPrescriptions, prescriptionAddDayMultiple);
		// 处方药销售排名
		prescriptionRankService.batchService(storeRankList, storeTopMultiple);
		// 药品排名
		medicineRankService.batchService(medicineRankList, medicineTopMultiple);
		// 地图数据
		areaStoreService.batchService(storeMapList, areaStoreMultiple);
		log.info("写数据到远程数据库结束");
		log.info("写数据到远程数据库，花费时间：{}", System.currentTimeMillis() - startTime);

		log.info("底部数据刷新任务结束");
		log.info("消耗时间：{}", System.currentTimeMillis() - startTime);
	}
}
